Struct tokio_util::io::StreamReader
source · [−]pub struct StreamReader<S, B> { /* private fields */ }Expand description
Convert a Stream of byte chunks into an AsyncRead.
This type performs the inverse operation of ReaderStream.
Example
use bytes::Bytes;
use tokio::io::{AsyncReadExt, Result};
use tokio_util::io::StreamReader;
// Create a stream from an iterator.
let stream = tokio_stream::iter(vec![
Result::Ok(Bytes::from_static(&[0, 1, 2, 3])),
Result::Ok(Bytes::from_static(&[4, 5, 6, 7])),
Result::Ok(Bytes::from_static(&[8, 9, 10, 11])),
]);
// Convert it to an AsyncRead.
let mut read = StreamReader::new(stream);
// Read five bytes from the stream.
let mut buf = [0; 5];
read.read_exact(&mut buf).await?;
assert_eq!(buf, [0, 1, 2, 3, 4]);
// Read the rest of the current chunk.
assert_eq!(read.read(&mut buf).await?, 3);
assert_eq!(&buf[..3], [5, 6, 7]);
// Read the next chunk.
assert_eq!(read.read(&mut buf).await?, 4);
assert_eq!(&buf[..4], [8, 9, 10, 11]);
// We have now reached the end.
assert_eq!(read.read(&mut buf).await?, 0);
If the stream produces errors which are not std::io::Error,
the errors can be converted using StreamExt to map each
element.
use bytes::Bytes;
use tokio::io::AsyncReadExt;
use tokio_util::io::StreamReader;
use tokio_stream::StreamExt;
// Create a stream from an iterator, including an error.
let stream = tokio_stream::iter(vec![
Result::Ok(Bytes::from_static(&[0, 1, 2, 3])),
Result::Ok(Bytes::from_static(&[4, 5, 6, 7])),
Result::Err("Something bad happened!")
]);
// Use StreamExt to map the stream and error to a std::io::Error
let stream = stream.map(|result| result.map_err(|err| {
std::io::Error::new(std::io::ErrorKind::Other, err)
}));
// Convert it to an AsyncRead.
let mut read = StreamReader::new(stream);
// Read five bytes from the stream.
let mut buf = [0; 5];
read.read_exact(&mut buf).await?;
assert_eq!(buf, [0, 1, 2, 3, 4]);
// Read the rest of the current chunk.
assert_eq!(read.read(&mut buf).await?, 3);
assert_eq!(&buf[..3], [5, 6, 7]);
// Reading the next chunk will produce an error
let error = read.read(&mut buf).await.unwrap_err();
assert_eq!(error.kind(), std::io::ErrorKind::Other);
assert_eq!(error.into_inner().unwrap().to_string(), "Something bad happened!");
// We have now reached the end.
assert_eq!(read.read(&mut buf).await?, 0);
Implementations
sourceimpl<S, B, E> StreamReader<S, B>where
S: Stream<Item = Result<B, E>>,
B: Buf,
E: Into<Error>,
impl<S, B, E> StreamReader<S, B>where
S: Stream<Item = Result<B, E>>,
B: Buf,
E: Into<Error>,
sourcepub fn into_inner_with_chunk(self) -> (S, Option<B>)
pub fn into_inner_with_chunk(self) -> (S, Option<B>)
Consumes this StreamReader, returning a Tuple consisting
of the underlying stream and an Option of the internal buffer,
which is Some in case the buffer contains elements.
sourceimpl<S, B> StreamReader<S, B>
impl<S, B> StreamReader<S, B>
sourcepub fn get_ref(&self) -> &S
pub fn get_ref(&self) -> &S
Gets a reference to the underlying stream.
It is inadvisable to directly read from the underlying stream.
sourcepub fn get_mut(&mut self) -> &mut S
pub fn get_mut(&mut self) -> &mut S
Gets a mutable reference to the underlying stream.
It is inadvisable to directly read from the underlying stream.
sourcepub fn get_pin_mut(self: Pin<&mut Self>) -> Pin<&mut S>
pub fn get_pin_mut(self: Pin<&mut Self>) -> Pin<&mut S>
Gets a pinned mutable reference to the underlying stream.
It is inadvisable to directly read from the underlying stream.
sourcepub fn into_inner(self) -> S
pub fn into_inner(self) -> S
Consumes this BufWriter, returning the underlying stream.
Note that any leftover data in the internal buffer is lost.
If you additionally want access to the internal buffer use
into_inner_with_chunk.